from typing import Callable, List, Optional, Union
import typer

from rich.console import Console
from rich.progress import Progress
from rich.table import Table

from blackneedles.connection import AnyModel


def print_table(
    ctx: typer.Context,
    data: List[AnyModel],
    attributes: List[Union[str, Callable[[AnyModel], str], str]],
    headers: List[str],
    title: Optional[str] = None,
) -> None:
    with Progress() as progress:
        if ctx.obj.table_style == "rich":
            table = Table(title=title)
            task = progress.add_task("Loading...", total=len(data))
        else:
            table = Table(
                title=title,
                box=None,
                show_header=True,
                show_edge=False,
                show_lines=False,
            )
            task = None

        for header in headers:
            table.add_column(header)
        for row in data:
            values = []
            for val in attributes:
                if not isinstance(val, str):
                    values.append(val(row))
                else:
                    values.append(getattr(row, val))
            table.add_row(*values)
            if task is not None:
                progress.advance(task)
    console = Console()
    console.print(table)



import configparser
import os
from typing import Any, Iterator, List, Optional, Sequence, Type, TypeVar

from pydantic import BaseModel
from snowflake.snowpark.row import Row
from snowflake.snowpark import Session
from snowflake.snowpark.exceptions import SnowparkSessionException
from snowflake.snowpark.context import get_active_session
from threading import local

threadlocal_data = local()

AnyModel = TypeVar("AnyModel", bound=BaseModel)


class Database:
    def __init__(self, session: Optional[Session] = None) -> None:
        if session is not None:
            self.session = session
            return
        try:
            self.session = get_active_session()
            return
        except SnowparkSessionException:
            pass
        self.config = {}
        config = configparser.ConfigParser()
        config.read(os.path.expanduser("~/.snowsql/config"))
        self.config = dict(
            account=os.getenv("SNOWFLAKE_ACCOUNT")
            or config["connections"]["accountname"],
            user=os.getenv("SNOWFLAKE_USER") or config["connections"]["username"],
            password=os.getenv("SNOWFLAKE_PASSWORD")
            or config["connections"]["password"],
            warehouse=os.getenv("SNOWFLAKE_WAREHOUSE")
            or config["connections"].get("warehousename"),
            database=os.getenv("SNOWFLAKE_DATABASE")
            or config["connections"].get("dbname"),
            schema=os.getenv("SNOWFLAKE_SCHEMA")
            or config["connections"].get("schemaname"),
            role=os.getenv("SNOWFLAKE_ROLE") or config["connections"].get("rolename"),
        )
        session_builder = Session.builder.config("connection_name", "default")
        session_builder.configs(self.config)
        self.session = session_builder.create()

    @classmethod
    def get_instance(self) -> "Database":
        if not hasattr(threadlocal_data, "database_instance"):
            threadlocal_data.database_instance = Database()
        return threadlocal_data.database_instance

    @classmethod
    def set_instance(self, instance: "Database") -> None:
        threadlocal_data.database_instance = instance

    def get_rows(self, sql: str, params: Optional[Sequence[Any]] = None) -> List[Row]:
        return self.session.sql(sql, params).collect()

    def query(
        self,
        model: Type[AnyModel],
        sql: str,
        params: Optional[Sequence[Any]] = None,
    ) -> Iterator[AnyModel]:
        for row in self.session.sql(sql, params).collect():
            yield model.model_validate(row.as_dict())

    def use_database(self, database_name: str) -> None:
        self.session.use_database(database_name)
        self.session.use_schema("__blackneedles__")



import datetime
from typing import Callable, Iterable, Optional
from pydantic import BaseModel

from blackneedles.connection import Database


class ComputePool(BaseModel):
    name: str
    state: str
    min_nodes: int
    max_nodes: int
    instance_family: str
    num_services: int
    num_jobs: int
    auto_suspend_secs: int
    auto_resume: bool
    active_nodes: int
    idle_nodes: int
    created_on: datetime.datetime
    resumed_on: Optional[datetime.datetime]
    updated_on: Optional[datetime.datetime]
    owner: str
    comment: Optional[str]
    is_exclusive: bool
    application: Optional[str]

    class Config:
        frozen = True

    class objects:
        @classmethod
        def all(cls) -> Iterable["ComputePool"]:
            """List all services from the given database"""
            db = Database.get_instance()
            return db.query(ComputePool, "SHOW COMPUTE POOLS")

        @classmethod
        def filter(
            self,
            callable: Callable[["ComputePool"], bool],
        ) -> Iterable["ComputePool"]:
            """Filter compute pool by the given callable"""
            for service in self.all():
                if callable(service):
                    yield service



import typer

from blackneedles.console import print_table
from blackneedles.models.compute_pool import ComputePool

app = typer.Typer(short_help="Manage compute pools in Snowpark Container Services.")


@app.command("list")
def list_all(
    ctx: typer.Context,
):
    """Gives the list of compute pools and its current status"""
    compute_pool = list(ComputePool.objects.all())

    print_table(
        ctx,
        compute_pool,
        [
            "name",
            "state",
            lambda c: f"{c.min_nodes} / {c.max_nodes}",
            lambda c: str(c.active_nodes),
            "instance_family",
            lambda c: f"{c.num_services} / {c.num_jobs}",
            "owner",
            "application",
        ],
        [
            "Name",
            "State",
            "# of nodes",
            "Active nodes",
            "# Services / Jobs" "Types",
            "Num Services",
            "Owner",
            "App",
        ],
    )
